/*
* Copyright (c) 2006-2009 Erin Catto http://www.gphysics.com
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#ifndef B2_POLYGON_SHAPE_H
#define B2_POLYGON_SHAPE_H

#include <Box2D/Collision/Shapes/b2Shape.h>

/// A convex polygon. It is assumed that the interior of the polygon is to
/// the left of each edge.
/// Polygons have a maximum number of vertices equal to b2_maxPolygonVertices.
/// In most cases you should not need many vertices for a convex polygon.
class b2PolygonShape : public b2Shape
{
public:
	b2PolygonShape();

	/// Implement b2Shape.
	b2Shape* Clone(b2BlockAllocator* allocator) const;

	/// @see b2Shape::GetChildCount
	int32 GetChildCount() const;

	/// Copy vertices. This assumes the vertices define a convex polygon.
	/// It is assumed that the exterior is the the right of each edge.
	/// The count must be in the range [3, b2_maxPolygonVertices].
	void Set(const b2Vec2* vertices, int32 vertexCount);

	/// Build vertices to represent an axis-aligned box.
	/// @param hx the half-width.
	/// @param hy the half-height.
	void SetAsBox(float32 hx, float32 hy);

	/// Build vertices to represent an oriented box.
	/// @param hx the half-width.
	/// @param hy the half-height.
	/// @param center the center of the box in local coordinates.
	/// @param angle the rotation of the box in local coordinates.
	void SetAsBox(float32 hx, float32 hy, const b2Vec2& center, float32 angle);

	/// @see b2Shape::TestPoint
	bool TestPoint(const b2Transform& transform, const b2Vec2& p) const;

	/// Implement b2Shape.
	bool RayCast(b2RayCastOutput* output, const b2RayCastInput& input,
					const b2Transform& transform, int32 childIndex) const;

	/// @see b2Shape::ComputeAABB
	void ComputeAABB(b2AABB* aabb, const b2Transform& transform, int32 childIndex) const;

	/// @see b2Shape::ComputeMass
	void ComputeMass(b2MassData* massData, float32 density) const;

	/// Get the vertex count.
	int32 GetVertexCount() const { return m_vertexCount; }

	/// Get a vertex by index.
	const b2Vec2& GetVertex(int32 index) const;
	
	static bool CheckVertexDirection(b2Vec2* v, int numVertices)
	{
		if(numVertices > 2) 
		{
			int wind = 0;
			int i = 0;
			
			while(wind == 0 && i < (numVertices - 2)) 
			{
				wind = v[i].winding(v[i + 1], v[i + 2]);
				++i;
			}
			
			if(wind < 0) 
			{
				return false;
			}
		}
		
		return true;
	}
	
	static bool EnsureCorrectVertexDirection(b2Vec2* vertices, int numVertices)
	{
		if(!b2PolygonShape::CheckVertexDirection(vertices, numVertices)) 
		{
			b2PolygonShape::ReverseVertices(vertices, numVertices);
			return false;
		}
		
		return true;
	}
	
	static void ReverseVertices(b2Vec2* v, int numVertices)
	{
		int low = 0;
		int high = numVertices - 1;
		int tmp;
		
		while(high > low) 
		{
			tmp = v[low].x;
			v[low].x = v[high].x;
			v[high].x = tmp;
			tmp = v[low].y;
			v[low].y = v[high].y;
			v[high].y = tmp;
			++low;
			--high;
		}
	}
	
	static b2PolygonShape* Decompose(float* v, int numVertices)
	{
		b2PolygonShape* result = new b2PolygonShape[numVertices];

		if(numVertices <= 2) 
		{
			return result;
		}
		
		//TODO?
		//lib.b2PolygonShape_Decompose(v);
		
		/*var p:Array = b2Base.getArr();
		
		for(int i = 0; i < p.length; ++i) 
		{
			result[i] = new b2PolygonShape(p[i]);
		}*/
		
		/*
		 
		 b2Polygon p;
		 b2Polygon results[100];
		 asm("%0 vt.get(%1)[0].length / 2;" : "=r" (p.nVertices) : "r" (args));	
		 if(p.nVertices <= 2) {
		 return AS3_Null();
		 }
		 int i, j;
		 p.x = new float32[p.nVertices];
		 p.y = new float32[p.nVertices];
		 asm("var v:Vector.<Number> = vt.get(%0)[0];" : : "r" (args));	
		 for(i = 0; i < p.nVertices; ++i) {
		 asm("%0 v[%1 * 2]" : "=r" (p.x[i]) : "r" (i));
		 asm("%0 v[%1 * 2 + 1]" : "=r" (p.y[i]) : "r" (i));
		 }
		 TraceEdge(&p);
		 int r = DecomposeConvex(&p, results, 100); // Arbitrary maximum
		 asm("arr = [];");
		 b2PolygonShape* s;
		 for(i = 0; i < r; ++i) {
		 s = new b2PolygonShape();
		 asm("arr.push(%0)" : : "r" (s));
		 for(j = 0; j < results[i].nVertices; ++j) {
		 s->m_vertices[j].x = results[i].x[j];
		 s->m_vertices[j].y = results[i].y[j];
		 }
		 s->Set(s->m_vertices, results[i].nVertices);
	}
	return AS3_Null();
		 
		 */
		
		return result;
	}
	
	b2Vec2 m_centroid;
	b2Vec2 m_vertices[b2_maxPolygonVertices];
	b2Vec2 m_normals[b2_maxPolygonVertices];
	int32 m_vertexCount;
};

inline b2PolygonShape::b2PolygonShape()
{
	m_type = e_polygon;
	m_radius = b2_polygonRadius;
	m_vertexCount = 0;
	m_centroid.SetZero();
}

inline const b2Vec2& b2PolygonShape::GetVertex(int32 index) const
{
	b2Assert(0 <= index && index < m_vertexCount);
	return m_vertices[index];
}

#endif
