/*
 * ----------------------------------------------------------------------------------
 * Copyright (c) 2008-2011, Stencyl, LLC
 * ----------------------------------------------------------------------------------
 * All rights reserved.
 * ----------------------------------------------------------------------------------
 */

#import <Foundation/Foundation.h>
#import "Sparrow.h" 

@class Actor;
@class ActorType;
@class Behavior;
@class Collision;
@class GroupDef;
@class Region;
@class Scene;
@class Transition;
@class Runnable;
@class Script;
@class SoundChannel;
@class NSURLConnection;

#ifndef TASK_H
#define TASK_H
typedef void (^Task)(Runnable* parent, Script* self);
#endif

#ifndef URLTASK_H
#define URLTASK_H
typedef void (^URLTask)(NSString* data, Script* self);
#endif

#ifndef IAPTASK_H
#define IAPTASK_H
typedef void (^IAPTask)(BOOL success, Script* self);
#endif

#ifndef CALLBACK1_H
#define CALLBACK1_H
typedef void (^SimpleCallback)(NSMutableArray* list, Script* self);
#endif

#ifndef CALLBACK2_H
#define CALLBACK2_H
typedef void (^DrawingCallback)(SPRenderSupport* g, int x, int y, BOOL screen, NSMutableArray* list, Script* self);
#endif

#ifndef CALLBACK3_H
#define CALLBACK3_H
typedef void (^CollisionCallback)(Collision* c, NSMutableArray* list, Script* self);
#endif

#ifndef CALLBACK4_H
#define CALLBACK4_H
typedef void (^SimpleActorCallback)(Actor* a, NSMutableArray* list, Script* self);
#endif

#ifndef CALLBACK5_H
#define CALLBACK5_H
typedef void (^ActorPositionCallback)(Actor* a, BOOL enteredScreen, BOOL exitedScreen, BOOL enteredScene, BOOL exitedScene, NSMutableArray* list, Script* self);
#endif

#ifndef CALLBACK6_H
#define CALLBACK6_H
typedef void (^PropertyCallback)(NSString* propertyName, NSMutableArray* list, Script* self);
#endif

#ifndef CALLBACK7_H
#define CALLBACK7_H
typedef void (^KeyCallback)(BOOL pressed, BOOL released, NSMutableArray* list, Script* self);
#endif

#ifndef CALLBACK8_H
#define CALLBACK8_H
typedef void (^TouchCallback)(int state, NSMutableArray* list, Script* self);
#endif

//----------------------------------------------------------------------------------
// Interface
//----------------------------------------------------------------------------------

@interface Script : NSObject 
{
	@public
        Behavior* mWrapper;
        Actor *mActor;
        BOOL isActor;
    
        NSMutableDictionary* mPropertyChangeListeners;
}

+ (Script*)script;

- (void)destroy;

- (void)mountActor:(Actor*)actor;
- (void)doesCustomDrawing;
- (void)handlesCollisions;

- (void)insertIntoList:(NSMutableArray*)list atIndex:(int)index withObject:(id)obj;
- (BOOL)sameAs:(id)one two:(id)two;
- (BOOL)sameAsOthers:(id)one otherOne:(id)otherOne otherTwo:(id)otherTwo;

//Conversions
- (float)asNumber:(id)obj;
- (NSString*)asString:(id)obj;
- (BOOL)asBoolean:(id)obj;

//Utils
+ (int)randomIntBetween:(int)minValue to:(int)maxValue;
+ (float)randomFloat;
+ (NSString*)substring:(NSString*)s start:(int)start end:(int)end;
+ (int)searchString:(NSString*)haystack for:(NSString*)needle;

//Flow
- (void)load;
- (void)update;
- (void)render:(SPRenderSupport*)g x:(int)x y:(int)y;

//Regions
- (Region*)getLastCreatedRegion;
- (NSDictionary*)getAllRegions;
- (Region*)getRegion:(int)regionID;
- (void)removeRegion:(int)regionID;	
- (Region*)createBoxRegion:(int)x y:(int)y w:(int)w h:(int)h;		
- (Region*)createCircularRegion:(int)x y:(int)y r:(int)r;	
- (BOOL)isInRegion:(Actor*)a r:(Region*)r;
- (NSArray*)getActorsInRegion:(Region*)r;

//Terrain Region

//Behavior Status
- (BOOL)sceneHasBehavior:(NSString*)behaviorName;
- (void)enableBehaviorForScene:(NSString*)behaviorName;
- (void)disableBehaviorForScene:(NSString*)behaviorName;
- (BOOL)isBehaviorEnabledForScene:(NSString*)behaviorName;
- (void)disableThisBehavior;

//Messaging
- (void)shout:(NSString*)msg;
- (id)saytoBehavior:(NSString*)recipient msg:(NSString*)msg withArgs:(NSArray*)args;
- (void)setAttributeForBehavior:(NSString*)behaviorName att:(NSString*)attName withValue:(void*)value;
- (id)getAttributeForBehavior:(NSString*)behaviorName att:(NSString*)attName;
- (void)shoutToScene:(NSString*)msg;
- (id)sayToSceneBehavior:(NSString*)recipient msg:(NSString*)msg withArgs:(NSArray*)args;
- (void)setAttributeForSceneBehavior:(NSString*)behaviorName att:(NSString*)attName withValue:(void*)value;
- (id)getAttributeForSceneBehavior:(NSString*)behaviorName att:(NSString*)attName;

//Game Attributes
- (void)setGameAttribute:(NSString*)name value:(id)value;
- (id)getGameAttribute:(NSString*)name;

//Timing
- (Runnable*)createRunnable:(Task)task;
- (void)doLater:(int)delayInMillis task:(Runnable*)task;
- (void)doPeriodically:(int)periodInMillis task:(Runnable*)task;
- (void)executeRunnable:(Runnable*)task repeat:(BOOL)repeat time:(int)time;

//Scene
- (Scene*)getScene;
- (int)getCurrentScene;
- (int)getIDForScene:(NSString*)sceneName;
- (NSString*)getCurrentSceneName;
- (int)getSceneWidth;
- (int)getSceneHeight;
- (int)getTileWidth;
- (int)getTileHeight;

//Scene Switching
- (void)reloadCurrentScene:(Transition*)leave enter:(Transition*)enter;
- (void)switchScene:(int)sceneID leave:(Transition*)leave enter:(Transition*)enter;
- (Transition*)createFadeOut:(int)duration color:(int)color;
- (Transition*)createFadeIn:(int)duration color:(int)color;

//Tile Layers
- (void)showTileLayer:(int)layerID;
- (void)hideTileLayer:(int)layerID;
- (void)fadeTileLayer:(int)layerID alpha:(int)alpha duration:(int)duration easing:(NSString*)easing;

//Camera
- (int)getScreenX;
- (int)getScreenY;
- (Actor*)getCamera;

//Input
- (int)numTouches;
- (BOOL)beganTouch;
- (BOOL)activeTouch;
- (BOOL)draggedTouch;
- (BOOL)releasedTouch;
- (int)getTouchX;
- (int)getTouchY;
- (int)getTouchDownX;
- (int)getTouchDownY;
- (int)getTouchDragX;
- (int)getTouchDragY;
- (int)getTouchReleaseX;
- (int)getTouchReleaseY;

//Actor Creation
- (Actor*)getLastCreatedActor;
- (Actor*)createRecycledActor:(ActorType*)type x:(int)x y:(int)y layerConst:(int)layerConst;
- (void)recycleActor:(Actor*)a;
- (Actor*)createActor:(ActorType*)type x:(int)x y:(int)y layerConst:(int)layerConst;
- (void)createActorInNextScene:(ActorType*)type x:(int)x y:(int)y layerConst:(int)layerConst;
//Skip getImageForActorType

//Actor Getters
- (ActorType*)getActorTypeByName:(NSString*)name;
- (ActorType*)getActorType:(int)actorTypeID;
- (NSArray*)getAllActorTypes;
- (NSSet*)getActorsOfType:(ActorType*)type;
- (Actor*)getActor:(int)actorID;
- (id)getActorGroup:(int)groupID;

//Joints
#ifdef __cplusplus
- (b2Joint*)getLastCreatedJoint;
- (b2Body*)getGroundBody;
- (b2Joint*)getJoint:(int)jointID;
- (void)destroyJointByID:(int)jointID;
- (void)destroyJoint:(b2Joint*)j;
- (b2DistanceJoint*)createStickJoint:(b2Body*)one two:(b2Body*)two p1:(b2Vec2*)p1 p2:(b2Vec2*)p2;
- (b2RevoluteJoint*)createHingeJoint:(b2Body*)one two:(b2Body*)two pt:(b2Vec2*)pt;
- (b2LineJoint*)createSlidingJoint:(b2Body*)one two:(b2Body*)two dir:(b2Vec2*)dir;
#endif

//Physics
//getActorsInArea
//getActorsOnScreen
//getActorsAtPoint
//getActorsAlongLine
- (float)getGravityX;
- (float)getGravityY;
- (void)setGravity:(float)x y:(float)y;
//enableContinuousCollisions (combined with makeActorNotPassThroughTerrain)
- (void)makeActorNotPassThroughTerrain:(Actor*)actor;

//Sound
- (void)mute;
- (void)unmute;
- (SPSound*)getSound:(int)ID;
- (void)stopAllSounds;
- (void)setMasterVolume:(float)amount;
- (void)fadeInMasterVolume:(float)time;
- (void)fadeOutMasterVolume:(float)time;
- (SPSoundChannel*)playSound:(SPSound*)sound;
- (SPSoundChannel*)loopSound:(SPSound*)sound;

- (void)playSoundOnChannel:(SPSound*)sound channelNum:(int)channelNum;
- (void)loopSoundOnChannel:(SPSound*)sound channelNum:(int)channelNum;
- (void)stopSoundOnChannel:(int)channelNum;
- (void)pauseSoundOnChannel:(int)channelNum;
- (void)resumeSoundOnChannel:(int)channelNum;
- (void)setVolumeForChannel:(float)volume channelNum:(int)channelNum;
- (void)fadeInSoundOnChannel:(int)channelNum time:(float)time;
- (void)fadeOutSoundOnChannel:(int)channelNum time:(float)time;

//Web Services
- (void)visitURL:(NSString*)url;
- (void)httpRequest:(NSString*)URL success:(URLTask)successBlock_;
- (void)postRequest:(NSString*)URL params:(NSString*)params success:(URLTask)successBlock_;

//Other Services
- (void)setIconBadgeNumber:(int)number;

//GUI (unused)

//Debug
- (void)enableDebugDrawing:(UIColor*)debugColor;
- (void)disableDebugDrawing;

//Utils
- (int)getFPS;
- (void)setFPS:(int)desiredFPS;
- (int)getStepSize;
//enableGlobalSleeping
//disableGlobalSleeping

//Background (set Scroll Speed)
//setScrollSpeedForBackground

//Eye Candy (not in iOS engine)
//Terrain Changer (not in iOS engine)

//Fonts
- (NSString*) getFont:(int)ID;

//Global
//pauseAll, unpauseAll -> specific to Flixel
- (int)getScreenWidth;
- (int)getScreenHeight;
- (void)setOffscreenTolerance:(int)top left:(int)left bottom:(int)bottom right:(int)right;
- (BOOL)isTransitioning;
//setTimeScale
//randomFloat -> SPUtils
//randomInt -> SPUtils
//tween, tweenNumber, abortTween - No equivalent?

//Saving (only local)
- (BOOL)saveGame;
- (BOOL)loadGame;

//User Info (not in iOS engine)
//Kongregate (not in iOS engine)

- (void)simulateKeyPress:(NSString*)abstractKey;
- (void)simulateKeyRelease:(NSString*)abstractKey;

- (BOOL)isKeyDown:(NSString*)abstractKey;
- (BOOL)isKeyPressed:(NSString*)abstractKey;
- (BOOL)isKeyReleased:(NSString*)abstractKey;

//Graphics
- (void)toScreenSpace;
- (void)toActorSpace;

//IAP
- (void)buyProduct:(NSString*)productID withCallback:(IAPTask)callback_;

//Events
- (void)addWhenUpdatedListener:(Actor*)a func:(SimpleCallback)func_;
- (void)addWhenDrawingListener:(Actor*)a func:(DrawingCallback)func_;

- (void)addCollisionListener:(Actor*)a func:(CollisionCallback)func_;
- (void)addSceneCollisionListener:(id)obj obj2:(id)obj2 func:(CollisionCallback)func_;

- (void)addWhenCreatedListener:(Actor*)a func:(SimpleCallback)func_;
- (void)addWhenTypeGroupCreatedListener:(id)obj func:(SimpleActorCallback)func_;

- (void)addWhenKilledListener:(Actor*)a func:(SimpleCallback)func_;
- (void)addWhenTypeGroupKilledListener:(id)obj func:(SimpleActorCallback)func_;

- (void)addActorPositionListener:(Actor*)a func:(ActorPositionCallback)func_;
- (void)addActorTypeGroupPositionListener:(id)obj func:(ActorPositionCallback)func_;

- (void)addActorEntersRegionListener:(Region*)r func:(SimpleActorCallback)func_;
- (void)addActorExitsRegionListener:(Region*)r func:(SimpleActorCallback)func_;

- (void)addSoundListener:(id)sound func:(SimpleCallback)func_;
- (void)addSoundChannelListener:(int)channelNum func:(SimpleCallback)func_;

- (void)addKeyPressedListener:(NSString*)key func:(KeyCallback)func_;
- (void)addTouchPressedListener:(SimpleCallback)func_;
- (void)addTouchDraggedListener:(SimpleCallback)func_;
- (void)addTouchReleasedListener:(SimpleCallback)func_;
- (void)addTouchActorListener:(Actor*)a func:(TouchCallback)func_;
- (void)addSwipedListener:(SimpleCallback)func_;

//----------------------------------------------------------------------------------
// Properties
//----------------------------------------------------------------------------------

@property (nonatomic, assign) Behavior* wrapper;
@property (nonatomic, assign) Actor* actor;
@property (nonatomic, assign) NSMutableDictionary* propertyChangeListeners;

@end
